<?php

/*
 * Copyright (C) 2017		 Oscss-Shop       <support@oscss-shop.fr>.
 *
 * This program is free software; you can redistribute it and/or modifyion 2.0 (the "License");
 * it under the terms of the GNU General Public License as published bypliance with the License.
 * the Free Software Foundation; either version 3 of the License, or
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 * or see https://www.gnu.org/
 */
require_once DOL_DOCUMENT_ROOT . '/admin/dolistore/class/dolistore.class.php';
include_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';
if (!class_exists('PrestaShopWebservice')) { // We keep this because some modules add this lib too into a different path. This is to avoid "Cannot declare class PrestaShopWebservice" errors.
    include_once DOL_DOCUMENT_ROOT . '/admin/dolistore/class/PSWebServiceLibrary.class.php';
}

/**
 * Class Dolistore
 */
class Dolistoreoscss extends Dolistore {

    /**
     * Constructor
     *
     * @param	boolean		$debug		Enable debug of request on screen
     */
    public function __construct($api, $debug = false) {
        global $conf, $langs;

        $this->url = dol_buildpath('/framework/admin/index.php', 2) . '?page=catalog'; // DOL_URL_ROOT.'/admin/modules.php?mode=marketplace';
        $this->shop_url = 'https://public.oscss-shop.fr/redirect/';
        $this->debug_api = $debug;
        $this->napi = $api;

        $langtmp = explode('_', $langs->defaultlang);
        $lang = $langtmp[0];
        $lang_array = array('en' => 1, 'fr' => 2, 'es' => 3, 'it' => 4, 'de' => 5); // Into table ps_lang of Prestashop - 1
        if (!in_array($lang, array_keys($lang_array)))
            $lang = 'en';
        $this->lang = $lang_array[$lang];
    }

    /**
     * Load data from remote Dolistore market place.
     * This fills ->categories
     *
     * @return	void
     */
// 	public function getRemoteCategories()
// 	{
// 	    global $conf;
// 
// 	    try {
// 	        $this->api = new PrestaShopWebservice($conf->global->MAIN_MODULE_DOLISTORE_API_SRV, $conf->global->MAIN_MODULE_DOLISTORE_API_KEY, $this->debug_api);
// 	        dol_syslog("Call API with MAIN_MODULE_DOLISTORE_API_SRV = ".$conf->global->MAIN_MODULE_DOLISTORE_API_SRV);
// 	        // $conf->global->MAIN_MODULE_DOLISTORE_API_KEY is for the login of basic auth. There is no password as it is public data.
// 
// 	        // Here we set the option array for the Webservice : we want categories resources
// 	        $opt              = array();
// 	        $opt['resource']  = 'categories';
// 	        $opt['display']   = '[id,id_parent,nb_products_recursive,active,is_root_category,name,description]';
// 	        $opt['sort']      = 'id_asc';
// 
// 	        // Call
// 	        dol_syslog("Call API with opt = ".var_export($opt, true));
// 	        $xml              = $this->api->get($opt);
// 	        $this->categories = $xml->categories->children();
// 	    } catch (PrestaShopWebserviceException $e) {
// 	        // Here we are dealing with errors
// 	        $trace = $e->getTrace();
// 	        if ($trace[0]['args'][0] == 404) die('Bad ID');
// 	        elseif ($trace[0]['args'][0] == 401) die('Bad auth key');
// 	        else
// 	        {
// 	            print 'Can not access to '.$conf->global->MAIN_MODULE_DOLISTORE_API_SRV.'<br>';
// 	            print $e->getMessage();
// 	        }
// 	    }
// 	}

    /**
     * Load data from remote Dolistore market place.
     * This fills ->products
     *
     * @param 	array 	$options	Options. If 'categorie' is defined, we filter products on this category id
     * @return	void
     */
    public function getRemoteProducts($options = array('start' => 0, 'end' => 10, 'per_page' => 50, 'categorie' => 0, 'search' => '')) {
        global $conf;

        $this->start = $options['start'];
        $this->end = $options['end'];
        $this->per_page = $options['per_page'];
        $this->categorie = $options['categorie'];
        $this->search = $options['search'];

        if ($this->end == 0) {
            $this->end = $this->per_page;
        }

        try {
            $res = $this->napi->GetCatalog($this->search);

            $this->products = $res;
        } catch (Exception $e) {
            // Here we are dealing with errors
            $trace = $e->getTrace();
            if ($trace[0]['args'][0] == 404)
                die('Bad ID');
            elseif ($trace[0]['args'][0] == 401)
                die('Bad auth key');
            else {
                print 'Can not access to ' . $conf->global->MAIN_MODULE_DOLISTORE_API_SRV . '<br>';
                print $e->getMessage();
            }
        }
    }

    // phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps

    /**
     * Return tree of Dolistore categories. $this->categories must have been loaded before.
     *
     * @param 	int			$parent		Id of parent category
     * @return 	string
     */
    public function get_categories($parent = 0) {
        // phpcs:enable
        if (!isset($this->categories))
            die('not possible');
        if ($parent != 0) {
            $html = '<ul>';
        } else {
            $html = '';
        }

        $nbofcateg = count($this->categories);
        for ($i = 0; $i < $nbofcateg; $i++) {
            $cat = $this->categories[$i];
            if ($cat->is_root_category == 1 && $parent == 0) {
                $html .= '<li class="root"><h3 class="nomargesupinf"><a class="nomargesupinf link2cat" href="?page=catalog&categorie=' . $cat->id . '" ';
                $html .= 'title="' . dol_escape_htmltag(strip_tags($cat->description->language[$this->lang - 1])) . '">' . $cat->name->language[$this->lang - 1] . ' <sup>' . $cat->nb_products_recursive . '</sup></a></h3>';
                $html .= self::get_categories($cat->id);
                $html .= "</li>\n";
            } elseif (trim($cat->id_parent) == $parent && $cat->active == 1 && trim($cat->id_parent) != 0) { // si cat est de ce niveau
                $select = ($cat->id == $this->categorie) ? ' selected' : '';
                $html .= '<li><a class="link2cat' . $select . '" href="?page=catalog&categorie=' . $cat->id . '"';
                $html .= ' title="' . dol_escape_htmltag(strip_tags($cat->description->language[$this->lang - 1])) . '" ';
                $html .= '>' . $cat->name->language[$this->lang - 1] . ' <sup>' . $cat->nb_products_recursive . '</sup></a>';
                $html .= self::get_categories($cat->id);
                $html .= "</li>\n";
            }
        }

        if ($html == '<ul>') {
            return '';
        }
        if ($parent != 0) {
            return $html . '</ul>';
        } else {
            return $html;
        }
    }

    // phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps

    /**
     * Return list of product formated for output
     *
     * @return string			HTML output
     */
// 	public function get_products($categorie=0, $listmyaccount)
    public function GetProducts($categorie = 0, array $ListMyAccount, $avaible = false, $search = '') {
        global $conf;

        if (count($avaible) == 0)
            $avaible = false;



        // phpcs:enable
        global $langs, $conf;
        $html = "";
        $last_month = time() - (30 * 24 * 60 * 60);
        foreach ($this->products as $product) {

            $product = (object) $product;

            if (!is_array($avaible) || key_exists($product->reference, $avaible)) {

                $already = (key_exists($product->reference, $ListMyAccount) ? $ListMyAccount[$product->reference] : false );

                // check new product ?
                $newapp = '';
                if ($last_month < strtotime($product->date_add)) {
                    $newapp .= '<span class="newApp">' . $langs->trans('New') . '</span> ';
                }

                // check updated ?
                // check updated ?
                if (key_exists($product->reference, $ListMyAccount) && $this->version_compare($product->module_version, $ListMyAccount[$product->reference]->version) > 0) {
                    $newapp .= '<span class="updatedApp">' . $langs->trans('NewVersion') . '</span> ';
                } elseif ($last_month < strtotime($product->date_upd) && $newapp == '') {
                    $newapp .= '<span class="updatedApp">' . $langs->trans('Updated') . '</span> ';
                }

                // add image or default ?
                if ($product->id_default_image != '') {
                    $image_url = DOL_URL_ROOT . '/admin/dolistore/ajax/image.php?id_product=' . $product->id . '&id_image=' . $product->id_default_image . '&quality=home_default';
                } else
                    $image_url = DOL_URL_ROOT . '/admin/dolistore/img/NoImageAvailable.png';

                $images = '<a href="' . $image_url . '" class="documentpreview" target="_blank" mime="image/png" title="' . $product->name->language[$this->lang - 1] . ', ' . $langs->trans('Version') . ' ' . $product->module_version . '">';
                $images .= '<img src="' . $image_url . '" style="max-height:250px;max-width: 210px;" alt="" /></a>';
// 			} else {
// 				$images = '<img src="'.DOL_URL_ROOT.'/admin/dolistore/img/NoImageAvailable.png" />';
// 			}
                // free or pay ?
                if ($product->price > 0 /* &&  !isset($conf->{$product->reference}) */) {
                    $price = '<h3>' . price(price2num($product->price, 'MT'), 0, $langs, 1, -1, -1, 'EUR') . ' ' . $langs->trans("HT") . '</h3>';

// 			    if( !isset($conf->{$product->reference}) ) 
                    $download_link = '<a target="_blank" href="' . $this->shop_url . $product->id . '/' . $product->reference . '"><img width="32" src="' . DOL_URL_ROOT . '/admin/dolistore/img/follow.png" /></a>';
                } else {
                    $price = '<h3>' . $langs->trans('Free') . '</h3>';

                    $download_link = '';
                }


                if ($already != false || $product->price == 0) {
                    $download_link = '<a target="_blank" href="' . dol_buildpath('framework/admin/index.php', 2) . '?page=catalog&action=redownload&module=' . $product->reference . (($product->istype == 'greffon') ? '&parent=' . $product->greffonname : '') . '"><img width="32" src="' . DOL_URL_ROOT . '/admin/dolistore/img/Download-128.png" /></a>';

                    $download_link .= '<br><br><a target="_blank" href="' . $this->shop_url . $product->id . '"><img width="32" src="' . DOL_URL_ROOT . '/admin/dolistore/img/follow.png" /></a>';
                }

                if (empty($product->dolibarr_max))
                    $dolibarr_max = DOL_VERSION; //(substr(DOL_VERSION,0,2) + 1).'.0.0';
                    
//checking versions
                if ($this->version_compare($product->dolibarr_min, DOL_VERSION) <= 0) {
                    if ($this->version_compare($dolibarr_max, DOL_VERSION) >= 0) {
                        //compatible
                        $version = '<span class="compatible">' . $langs->trans('CompatibleUpTo', $dolibarr_max,
                                        $product->dolibarr_min, $dolibarr_max) . '</span>';
                        $compatible = '';
                    } else {
                        //never compatible, module expired
                        $version = '<span class="notcompatible">' . $langs->trans('NotCompatible', DOL_VERSION,
                                        $product->dolibarr_min, $dolibarr_max) . '</span>';
                        $compatible = 'NotCompatible';
                    }
                } else {
                    //need update
                    $version = '<span class="compatibleafterupdate">' . $langs->trans('CompatibleAfterUpdate', DOL_VERSION,
                                    $product->dolibarr_min, $dolibarr_max) . '</span>';
                    $compatible = 'NotCompatible';
                }

                //.'<br><a class="inline-block valignmiddle" target="_blank" href="'.$this->shop_url.$product->id.'"><span class="details button">'.$langs->trans("SeeInMarkerPlace").'</span></a>
                //output template
                $html .= '<tr class="app oddeven ' . $compatible . '">
                <td class="center" width="210"><div class="newAppParent">' . $newapp . $images . '</div></td>
                <td class="margeCote"><h2 class="appTitle">' . $product->reference . '  (' . $product->module_version . ')'//->name->language[$this->lang - 1]
                        . '<br/><small>' . $version . '</small></h2>
                    <small> ' . dol_print_date(dol_stringtotime($product->date_upd), 'dayhour') . ' - ' . $langs->trans('Ref') . ': ' . $product->reference . ' - ' . $langs->trans('Id') . ': ' . $product->id . '</small><br><br>' . $product->short_description_fr /* description_short->language[$this->lang - 1] */ . '</td>';
                // do not load if display none
                //$html .= '<td style="display:none;" class="long_description">'.$product->description->language[$this->lang - 1].'</td>';
                $html .= '<td class="margeCote center">' . $price . '</td>';
                $html .= '<td class="margeCote">' . $download_link . '</td>';
                $html .= '</tr>';
            }
        }
        return $html;
    }

}
