<?php
/* Copyright (C) 2016		 Oscss-Shop       <support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	@file /class/staticdico.class.php
 *	@group framework
 *	@class Staticdico
 *	@brief This class load a json and retrun key/value based on dic
 *	@example UsageStaticDico
 *		// load file \n
 *		dol_include_once('/framework/class/staticdico.class.php');  \n
 *		// new class \n
 *       $StaticDico = new \Staticdico($this->db); \n 
 *      // load dico \n
 *     $StaticDico->SetDico(dol_buildpath('/edi/staticdico/7065PackageType.json',0));  \n \n
 *    
 *   $gstype = $StaticDico->GetCode('7065PackageType', $type); \n
 *       
 *      $gscode = $StaticDico->GetId('7065PackageType', $code); \n
 *       
 *       $gslabel = $StaticDico->GetLabel('7065PackageType', $code); \n
 *	
 *		$gslabel = $StaticDico->GetFull('7065PackageType'); 
 */
class Staticdico
{

	/**
	 * __construct
	 *
	 * @return void
	 */
	function __construct()
	{
	}


	/**
	 * GetFull
	 *
	 * @param  string $type
	 * @param  mixed $fk_parent
	 * @return false Ko / string OK
	 */
	function GetFull($type, $fk_parent = false)
	{

		if (is_object($this->{$type})) {
			if ($fk_parent != false)
				return $this->GetChild($type, $fk_parent);
			else
				return $this->{$type};
		}

		return false;
	}

	/**
	 *	@fn GetChild()
	 *	@param string $type
	 *	@param string $key id of parent in fk_parent 
	 *	@return false Ko / string OK
	 */
	function GetChild($type, $key)
	{
		if (is_object($this->{$type})) {
			$childs = array();
			foreach ($this->{$type} as $row)
				if ($row['fk_parent'] == $key)
					$childs[] = $row;

			return $childs;
		}

		return false;
	}

	/**
	 *	@fn GetCode()
	 *	@param string $type
	 *	@param string $key
	 *	@return false Ko / string OK
	 */
	function GetCode($type, $key)
	{
		if (is_object($this->{$type})) {
			foreach ($this->{$type} as $row)
				if ($row['key'] == $key)
					return $row['code'];
		}

		return false;
	}

	/**
	 *	@fn GetId()
	 *	@param string $type
	 *	@param string $code
	 *	@return false Ko / string OK
	 */
	function GetId($type, $code)
	{
		if (is_object($this->{$type})) {
			foreach ($this->{$type} as $row)
				if ($row['code'] == $code)
					return $row['key'];
		}

		return false;
	}

	/**
	 *	@fn GetLabel()
	 *	@param string $type
	 *	@param string $code
	 *	@return false Ko / string OK
	 */
	function GetLabel($type, $code)
	{
		if (is_object($this->{$type})) {
			foreach ($this->{$type} as $row)
				if ($row['code'] == $code)
					return $row['label'];
		}

		return false;
	}

	/**
	 *	@fn SetDico()
	 *	@param string $path
	 *	@return none
	 */
	function SetDico($path)
	{
		if (substr($path, -4) != 'json')
			$path .= '.json';

		if (file_exists($path)) {

			$f = json_decode(file_get_contents($path), true);
			if (json_last_error() == JSON_ERROR_NONE) {

				$name = substr(basename($path), 0, -5);
				$this->{$name} = (object) $f;
			}
		}
	}
}

