<?php
/* Copyright (C) 2016		 Oscss-Shop       <support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */
dol_include_once('/framework/class/api.client.inc.php');
dol_include_once('/framework/class/apiregistration.class.php');
dol_include_once('/includes/odtphp/zip/pclzip/pclzip.lib.php');


/**
 *	@class updateOscssshop
 *	@brief Class for manage display in module list 
 *			Add link for registered module or link to download last version 
 *			Use SESSION for stock request API for list of module/version avaiable
 *			
 */
class updateOscssshop

{
    public $installDir;
    public $module;
    public $server              = 'https://dolistore.oscss-shop.fr/api/';
    public $api_key             = '5C6494B713AC6588BB98475528B4C';
    public $checked             = false;
    public $version             = '0.0.0';
    public $downloadId;
    public $greffonofmastername = '';
    
    public $cacheversion=array();
       
    /**
     * __construct
     *
     * @param  mixed $module
     * @return void
     */
    function __construct($module = FALSE)
    {
        global $langs;

        $langs->load('framework@framework');

        if ($module !== FALSE) {
            $this->setModule($module);
        }

        if (isset($_SESSION['cacheversion'])) {
            $this->cacheversion = $_SESSION['cacheversion'];
        }
    }	

    /**
     * @fn download($module = FALSE, $greffonofmastername=FALSE)
     * @brief this method load last version for one module and extract in dir ;
     * OR download new module / greffon and extract ressource
     * @param string/FALSE  $module for specifie ressource
     * @param  string/FALSE $greffonofmastername if module is greffon, precise parent of greffon
     * @return boolean true OK / False KO
     */
    public function download($module = FALSE, $greffonofmastername = FALSE)
    {
        if ($module !== FALSE) {
            $this->setModule($module);
        }
        if ($greffonofmastername !== FALSE) $this->greffonofmastername = $greffonofmastername;
        else $this->greffonofmastername = $this->module;

        if (!$this->checked) {
            $this->getVersion($this->module);
        }
        if ($this->version == '0.0.0') { // no version found
            global $conf;

            $path = dol_buildPath('/framework/core/build.json', 0);

            $build = json_decode(file_get_contents($path));

            if (is_object($build)) {
                // editor
                $this->editor_name = $build->editor->name;
                // For update linked
                $this->urlapi = $build->editor->api;
                $this->api_key = $build->editor->apikey;

                $this->cstname = "FRAMEWORKAPIKEYLINK" . strtoupper(str_replace(' ', '', trim($this->editor_name)));

                $apiregis = new ApiRegistration($build->editor->api, $build->editor->apikey, $conf->global->{$this->cstname});

                $result = $apiregis->getMyAccount();



                foreach ($result as $k => $r) {
                    $r = (array)$r;
                    if ($r['module'] == $this->module) {
                        $this->downloadId = $r['id_download'];
                        $this->version    = $r['version'];
                    }
                }
            }

            if ($this->version == '0.0.0')
            return false;
        }



        if (class_exists('apiClient')) {
            $api = new apiClient();
            $api->end_point = $this->server;
			$api->FixMode('download'); 
			
            
            $this->installDir = dol_buildpath("/$this->greffonofmastername/", 0);

            if (!file_exists($this->installDir)) {
                $this->installDir = str_replace("/$this->greffonofmastername/", "/custom/$this->greffonofmastername/", $this->installDir);
                @mkdir($this->installDir);
            }
            $zipFile          = "{$this->installDir}{$this->module}-{$this->version}.zip";
            $output           = false;
            
            
            
            $url = "minpackages/{$this->downloadId}/?api_key={$this->api_key}";

            $res = $api->send($url);



            if (file_exists($zipFile)) unlink($zipFile); // cleanup
                $size = file_put_contents($zipFile, $api->curl_response);


            if ($size > 0) {
                $zip  = new PclZip($zipFile);

                if (($list = $zip->listContent()) == 0) {
                    return FALSE;
                }
                // echo '<pre>';
                $zipPath = 'htdocs/custom';
                foreach ($list as $file) { // find zip structure
                    $matches     = null;
                    $returnValue = preg_match('/^(.*?)\\/' . $this->greffonofmastername . '*/i', $file['filename'], $matches);

                    if ($returnValue) {
                        $zipPath = $matches[1];
                        $output  = true;
                        break;
                    }
                }

                if ($output) {

                    $tmp = $this->ExtractZip($zip, $zipPath ); 
                    if( $tmp != true)
                        $tmp = $this->ExtractZip($zip, $zipPath ); 
                        
                    if( $tmp != true  ) 
                        setEventMessages('FileProtect',$tmp, 'errors');

                    $this->rrmdir($this->installDir . 'htdocs');

                        if ($zip->errorCode == 0) 
                            return true;
                }
                else 
                    setEventMessages('ErrorPath',array('ErrorPath'), 'errors');
            }
            else 
                setEventMessages('ErrorDownload',array('ErrorDownload'), 'errors');
        }

        return $output;
    }


    /**
        @fn ExtractZip()
        @param $zip object 
        @param $zipPath  string 
        @return true if OK or array file in error 
    */
    protected function ExtractZip($zip, $zipPath ){
        global $conf; 

        $res = $zip->extract(PCLZIP_OPT_PATH, $this->installDir, PCLZIP_OPT_REMOVE_PATH, $zipPath . '/' . $this->greffonofmastername);
        
        $tmp = []; 
        foreach($res as $row) 
            if($row['status'] == 'write_protected') {
                if( ! chmod( dol_buildPath( str_replace('htdocs', '', $row['stored_filename']), 0 ) , octdec(empty($conf->global->MAIN_UMASK)?'0664':$conf->global->MAIN_UMASK))  ) 
                    $tmp[] = 'FileProtect '.$row['stored_filename'];
            }
            

        if(count($tmp)>0 ) 
            return $tmp; 
        
        return true; 
    }
    
    /**
     * CacheVersion @brief mise en cache de la requete des versions dispo
     *
     * @return void
     */
    function CacheVersion()
    {
        global $conf;

        if (count($this->cacheversion) > 0 && isset($_SESSION['cacheversion'])  && count($_SESSION['cacheversion']) == count($this->cacheversion))
        return $this->cacheversion;

        if (file_exists(dol_buildPath('/' . $this->module . '/core/build.json', 0)))
        $path = dol_buildPath('/' . $this->module . '/core/build.json', 0);
        else
        $path = dol_buildPath('/framework/core/build.json', 0);

        $build = json_decode(file_get_contents($path));

        if (is_object($build)) {
            // editor
            $this->editor_name = $build->editor->name;
            // For update linked
            $this->urlapi = $build->editor->api;
            $this->api_key = $build->editor->apikey;

            $this->cstname = "FRAMEWORKAPIKEYLINK" . strtoupper(str_replace(' ', '', trim($this->editor_name)));

            $apiregis = new ApiRegistration($build->editor->api, $build->editor->apikey, $conf->global->{$this->cstname});

            $result = $apiregis->GetVersion();
            
            if($result == false)
                return false; 

            if(!is_object($result))
                $result = (object)$result; 


            if( count($result->data ) == 0 ) 
                return false;
			
        }


        
        $this->cacheversion = array();
        if (is_array($result->data) && $result->total  > 0) {

            foreach ($result->data as $k => $r) {

                $row = new stdClass();
                $r = (object)$r; 

                $row->checked    = true;
                $row->downloadId = $r->id_download;
                $row->version    = $r->version;

                $this->cacheversion[$r->module] = $row;
            }
        }

        $_SESSION['cacheversion'] = $this->cacheversion;

        return $this->cacheversion;
    }	
	
        
    /**
     * @fn getVersion($module = FALSE)
     * @brief
     * check level module in dolibarr cf MAIN_FEATURES_LEVEL
     *
     * @param
     * @return version string xx.xx.xx
     */
    function getVersion($module = FALSE)
    {
        global $conf;

        if ($module !== FALSE) {
            $this->setModule($module);
        }

        $result = $this->CacheVersion();

        if (is_array($result)) {

            foreach ($result as $k => $r) {

                if ($this->module == $k) {
                    $this->checked    = true;
                    $this->downloadId = $r->downloadId;
                    $this->version    = $r->version;

                    return $this->version;
                }
            }
        }

        return '0.0.0';
    }


    /**
	 *	@fn getGreffons()
	 *	@brief 
	 *	@param string $module or FALSE
     */
    function getGreffons($module = FALSE)
    {
        global $conf;

        if ($module !== FALSE) {
            $this->setModule($module);
        }
        // if ($this->checked) return $this->version;
        if (class_exists('apiClient')) {
            $api = new apiClient();
            $url = "{$this->server}greffon/?filter[code]={$this->module}&order_by=original_name&order=desc&api_key={$this->api_key}";
            $api->send($url);

            if ($api->decoded->total > 0) { // find something and return most recent version
                foreach ($api->decoded->data as $k => $r) {

                    $this->greffons[$r->code] = $r;
                }

                return $this->greffons;
            }
        }
    }

    /**
     * @fn setModule($module)
     * @brief Fix name of module
     */
    function setModule($module)
    {
		if(empty($module))
			$module = 'framework';
        return $this->module = $module;
    }

    /**
     * @fn setServer($server)
     * @brief Fix url of server
     */
    function setServer($server)
    {
        return $this->server = $server;
    }

    /**
     * @fn setKey($key)
     * @brief Fix url of server
     */
    function setKey($key)
    {
        return $this->api_key = substr($key, 6);
    }

    /**
	 *	@fn insertUpdateLink()
     */
    function insertUpdateLink(&$module, $langs)
    {
        if (!is_object($langs))
        global $langs;

        $langs->load('framework@framework');

        $distVer = $this->getVersion();

        if (
            !isset($this->cacheversion[strtolower($module->name)]) /*&& !$module->greffon*/
        )
        return '<a class="button" href="' . dol_buildPath('/' . (($module->greffon == 0) ? strtolower($module->name)  : strtolower($module->herit) ) . '/admin/index.php?page=registration&action=addmod&newmod=' . strtolower($module->name), 2) . '">' . $langs->trans("NotRegistered") . '</a> ';
        
        if (version_compare($distVer, $module->getVersion(), '>')) {

            return '<a class="button" href="' . dol_buildPath('/framework/update.php?module=' . strtolower($module->name) . (($module->greffon == 0) ? '' : '&parent=' . $module->herit . '&redirect=somewhere&path=' . urlencode('/' . $module->herit . '/admin/?page=greffon')), 2) . '">' . $langs->trans("UpdateTo", "$distVer") . ' - ' . $distVer . '</a> '; //.$langs->trans($module->description);
        }

        // return $module->description; 
    }

    /**
	 *	@fn rrmdir()
	 */
    function rrmdir($dir)
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (is_dir($dir . "/" . $object)) $this->rrmdir($dir . "/" . $object);
                    else unlink($dir . "/" . $object);
                }
            }
            rmdir($dir);
        }
    }
}
